"""LangSmith integration for OpenAI Agents SDK.

This module provides tracing support for the OpenAI Agents SDK.
"""

from ._openai_agents import OpenAIAgentsTracingProcessor

__all__ = ["OpenAIAgentsTracingProcessor"]
